'use strict';
{
    const C3 = self.C3;
    C3.Behaviors.Fader.Instance = class FaderInstance extends C3.SDKBehaviorInstanceBase {
        constructor(behInst, properties) {
            super(behInst);
			this.isRunning = true;
			this.fadeInTime = 0;
            this.waitTime = 0;
            this.fadeOutTime = 0;
			this.destroy = 0; // 0 = no, 1 = after fade out
			this.timeInStage = 0;
			this.stage = 0;
			this.maxOpacity = this._inst.GetWorldInfo().GetOpacity() || 1;
            if (properties) {
				this.isRunning = !!properties[0];
				this.fadeInTime = properties[1];
				this.waitTime = properties[2];
				this.fadeOutTime = properties[3];
				this.destroy = properties[4];
            }
			
            if (this.isRunning)
			{
				// Skip fade-in
				if (this.fadeInTime === 0) 
				{
                    this.stage = 1;
					
					// Skip wait
                    if (this.waitTime === 0)
                        this.stage = 2
                }
				// Otherwise we don't want it at default opacity for first tick. Set to 0 opacity.
				else 
				{
					this.stage = 0;	// 0 = fade in, 1 = wait, 2 = fade out, 3 = done
					this._inst.GetWorldInfo().SetOpacity(0);
                    this._runtime.UpdateRender();
                }
			}
            this._StartTicking()
        }
        Release() {
            super.Release()
        }
        SaveToJson() {
            return {
                "fit": this.fadeInTime,
                "wt": this.waitTime,
                "fot": this.fadeOutTime,
				"s": this.stage,
				"tis": this.timeInStage,
				"mo": this.maxOpacity
            }
        }
        LoadFromJson(o) {
            this.fadeInTime = o["fit"];
            this.waitTime = o["wt"];
            this.fadeOutTime = o["fot"];
			this.stage = o["s"];
			this.timeInStage = o["tis"];
			this.maxOpacity = o["mo"];
        }
		Tick() {
			const dt = this._runtime.GetDt(this._inst);
			this.inst = this._inst.GetWorldInfo();
			if (this.isRunning)
			{
				this.timeInStage += dt;
				
				// Stage 0: fade-in
				if (this.stage === 0)
				{
					this.inst.SetOpacity((this.timeInStage / this.fadeInTime) * this.maxOpacity);
					this._runtime.UpdateRender();

					// Fade-in completed
					if (this.inst.GetOpacity() >= this.maxOpacity)
					{
						this.inst.SetOpacity(this.maxOpacity);
						this.stage = 1;	// wait stage
						this.timeInStage = 0;
						
						// On fade-in end
						this.Trigger(C3.Behaviors.Fader.Cnds.OnFadeInEnd);
					}
				}
				
				// Stage 1: wait
				if (this.stage === 1)
				{
					// Wait time has elapsed
					if (this.timeInStage >= this.waitTime)
					{
						this.stage = 2;	// fade out stage
						this.timeInStage = 0;
						
						// On wait end
						this.Trigger(C3.Behaviors.Fader.Cnds.OnWaitEnd);
					}
				}
				
				// Stage 2: fade out
				if (this.stage === 2)
				{
					if (this.fadeOutTime !== 0)
					{
						this.inst.SetOpacity(this.maxOpacity - ((this.timeInStage / this.fadeOutTime) * this.maxOpacity));
						this._runtime.UpdateRender();
						
						// Fade-out completed
						if (this.inst.GetOpacity() <= 0)
						{
							this.inst.SetOpacity(0);
							this.stage = 3;	// done
							this.timeInStage = 0;
							
							// On fade-out end
							this.Trigger(C3.Behaviors.Fader.Cnds.OnFadeOutEnd);
							
							// End the fade out
							if (this.destroy === 0)
								this.isRunning = false;
							
							else if (this.destroy === 1)
								this._runtime.DestroyInstance(this._inst);
							
							else if (this.destroy === 2)
								this.stage = 0;
						}
					}
				}
			}
		}
		setupAndStartFade()
		{
			this.stage = 0;
			this.timeInStage = 0;
			// Skip fade-in
			if (this.fadeInTime === 0) {
				this.stage = 1;
				// Skip wait
				if (this.waitTime === 0)
					this.stage = 2;
			} else {
				this._runtime.UpdateRender();
			}
		}
        GetDebuggerProperties() {
            return [{
                title: "$" + this.GetBehaviorType().GetName(),
                properties: [{
                    name: "Fade in time",
                    value: this.fadeInTime,
                    onedit: v=>this.fadeInTime = v
                }, {
                    name: "Wait time",
                    value: this.waitTime,
                    onedit: v=>this.waitTime = v
                }, {
                    name: "Fade out time",
                    value: this.fadeOutTime,
                    onedit: v=>this.fadeOutTime = v
                }, {
                    name: "Stage",
                    value: [["fade-in", "wait", "fade-out", "done"][this.stage]]
                }]
            }]
        }
    }
};